<?php
/**
 * Created by PhpStorm.
 * User: heyu
 * Date: 2019-05-16
 * Time: 14:26
 */

namespace kwy\mybank;


use kwy\mybank\exception\MybankException;
use kwy\mybank\exception\MyBankNetException;
use kwy\mybank\exception\MybankPayingException;
use kwy\mybank\request\MybankApi;

class MybankPay extends MybankApi
{





    /**
     * 微信Js支付
     */
    public function wxJsPay(){

    }

    /**
     * 支付宝Js支付
     */
    public function aliJsPay(){

    }


    /**
     * @param $auth_code
     * @param $amount
     * @param $no
     * @param $body
     * @param $merchant_id
     * @param array $options
     * 'SubAppId'  微信公众号 appid
     *
     * @return MybankPayOrder
     * @throws MybankPayingException
     */
    public function microPay($auth_code,$amount,$no,$body,$merchant_id,$options=[]){
        $function='ant.mybank.bkmerchanttrade.pay';
        $biz_params=[
            'AuthCode'=>$auth_code,
            'OutTradeNo'=>$no,#系统流水号 唯一
            'Body'=>$body,
            'TotalAmount'=>$amount,#总金额  分
            'Currency'=>'CNY',
            'MerchantId'=>$merchant_id,
            'DeviceCreateIp'=>'115.29.165.48',
            'SettleType'=>'T1',
        ];
        if (!empty($options)){
            $biz_params=array_merge($biz_params,$options);
        }

        $auth_code_prefix=substr($auth_code,0,1);
        if ($auth_code_prefix=='1'){
            $biz_params['ChannelType']='WX';
        }elseif ($auth_code_prefix=='2'){
            $biz_params['ChannelType']='ALI';
        }
        $ret=$this->getBizResult($function,$biz_params);
        if (!empty($ret['CnlMsg'])){
            $ret['CnlMsg']=json_decode(base64_decode($ret['CnlMsg']));
        }

        if ($ret['RespInfo']['ResultStatus']=='F'){
            throw new \Exception($ret['RespInfo']['ResultMsg']);
        }elseif ($ret['RespInfo']['ResultStatus']=='U'){
            throw new MybankPayingException('用户支付中');
        }
        return new MybankPayOrder($ret);
    }

    /**
     * @param $amount
     * @param $no
     * @param $body
     * @param $merchant_id
     * @param $openid
     * @param $appid
     * @param bool $isMiniPrograme
     * @param string $ip 默认115.230.116.154
     * @param string $notifyUrl 默认网商预留地址
     * @return array 支付array
     * @throws \Exception
     */
    public function jsPay($amount,$no,$body,$merchant_id,$openid,$appid,$isMiniPrograme=false,$ip="",$notifyUrl=""){
        $function='ant.mybank.bkmerchanttrade.prePay';
        $isAli=is_numeric($openid);
        $biz_params=[
            'OutTradeNo'=>$no,
            'Body'=>$body,
            'TotalAmount'=>$amount,
            'Currency'=>'CNY',
            'MerchantId'=>$merchant_id,
            'ChannelType'=>$isAli?'ALI':'WX',
            'OpenId'=>$openid,
            'DeviceCreateIp'=>empty($ip)?"115.230.116.154":$ip,
            'ExpireExpress'=>'2',
            'SettleType'=>'T1',
        ];
        if (!empty($notifyUrl)){
            $biz_params['NotifyUrl']=$notifyUrl;
        }
        if ($isMiniPrograme){
            $biz_params['SubAppIdType']='MINI_PROGRAM';
        }
        if (!$isAli){
            $biz_params['SubAppId']=$appid;
        }
        $ret = $this->getBizResult($function, $biz_params);
        $arr_params_key=['CnlMsg'];
        foreach ($arr_params_key as $v){
            if (!empty($ret[$v])){
                $ret[$v]=json_decode(base64_decode($ret[$v]),true);
            }
        }
        if (!$isAli){
            if (!empty($ret['PayInfo'])){
                return json_decode($ret['PayInfo'],true);
            }elseif (!empty($ret['CnlMsg'])){
                if ($ret['CnlMsg']['result_code']=='SUCCESS'){
                    return json_decode($ret['CnlMsg']['wx_pay_data'],true);
                }else {
                    if ($ret['CnlMsg']['err_code']=='SUBAPPID_NOTMATCH'){
                        throw new \Exception('商户号未关联该公众号或小程序');
                    }else {
                        throw new \Exception($ret['CnlMsg']['err_code'].':'.$ret['CnlMsg']['err_code_des']);
                    }
                }
            }else {
                $exp_info='';
                if (!empty($ret['RespInfo']['ResultMsg'])){
                    $exp_info=$ret['RespInfo']['ResultMsg'].':'.$ret['RespInfo']['ResultCode'];
                }
                throw new \Exception('未知异常,'.$exp_info);
            }

        }else{
            if (!empty($ret['PrePayId'])){
                return ['tradeNO'=>$ret['PrePayId']];
            }elseif(!empty($ret['CnlMsg'])){
                throw new \Exception($ret['CnlMsg']['sub_code'].','.$ret['CnlMsg']['sub_msg']);
            }else{
                throw new \Exception($ret['RespInfo']['ResultMsg']);
            }
        }
    }

    /**
     * 单据查询
     * @param $mch_id
     * @param $no
     * @return MybankPayOrder
     * @throws MybankException
     */
    public function query($mch_id,$no){
        $function='ant.mybank.bkmerchanttrade.payQuery';
        $biz_params=[
            'MerchantId'=>$mch_id,
            'OutTradeNo'=>$no,
        ];

        try {
            $ret=$this->getBizResult($function, $biz_params);
        }catch (MyBankNetException $net_e){
            $exception=new MybankPayingException('用户支付中');
            throw $exception;
        }
        if ($ret['RespInfo']['ResultStatus']!=='S'||$ret['RespInfo']['ResultCode']!=='0000'){
            $e = new MybankException($ret['RespInfo']['ResultMsg']);
            $e->setResult($ret);
            throw $e;
        }

        $base64_decode_fields=['CnlMsg'];
        foreach ($ret as $k=>&$v){
            if (in_array($k,$base64_decode_fields)){
                $v=json_decode(base64_decode($v),true);
            }
        }
        switch ($ret['TradeStatus']){
            case 'succ':{
                $result=new MybankPayOrder($ret);
                return $result;
            }
            case 'fail':{
                $errmsg='支付失败';
                if (isset($ret['CnlMsg']['err_code_des'])){
                    $errmsg=$ret['CnlMsg']['err_code_des'];
                }
                throw new MybankException($errmsg,0,null,$ret);break;
            }
            case 'paying':{
                throw new MybankPayingException('用户支付中',0,null,$ret);break;
            }
            case 'closed':
            case 'cancel':{
                throw new MybankException('交易已关闭或撤销,请重新支付',0,null,$ret);break;
            }
            default:{
                throw new MybankException('未知交易结果,请联系技术处理,TradeStatus:'.$ret['TradeStatus'],0,null,$ret);
            }
        }

    }
}